from setuptools import setup, find_packages

import sys
import opencanary

requirements = [
    'Twisted==19.10.0',
    'pyasn1==0.4.5',
    'cryptography==3.0',
    'simplejson==3.16.0',
    'requests==2.21.0',
    'zope.interface==5.0.0',
    'PyPDF2==1.26.0',
    'fpdf==1.7.2',
    'passlib==1.7.1',
    'Jinja2==2.10.1',
    'ntlmlib==0.72',
    'bcrypt==3.1.7',
    'hpfeeds==3.0.0']

if sys.version_info.major < 3:
    requirements.append('wsgiref==0.1.2')

setup(
    name='opencanary',
    version=opencanary.__version__,
    url='http://www.thinkst.com/',
    author='Thinkst Applied Research',
    author_email='info@thinkst.com',
    description='OpenCanary daemon',
    long_description='A low interaction honeypot intended to be run on internal networks.',
    install_requires=requirements,
    setup_requires=[
        'setuptools_git'
    ],
    license='BSD',
    packages=find_packages(exclude='test'),
    scripts=['bin/opencanaryd','bin/opencanary.tac'],
    platforms='any',
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Framework :: Twisted",
        "Topic :: System :: Networking",
        "Topic :: Security",
        "Topic :: System :: Networking :: Monitoring",
        "Natural Language :: English",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Operating System :: POSIX :: BSD :: FreeBSD",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: BSD License",
    ],
)



from opencanary.modules import CanaryService

from twisted.application import internet
from twisted.internet.protocol import DatagramProtocol

from twisted.application.internet import UDPServer
from twisted.internet.address import IPv4Address

from twisted.internet import protocol

"""
    A log-only NTP server. It won't respond, but it will log attempts
    to trigger the MON_GETLIST_1 NTP commands, which is used for DDOS
    and network recon.
"""

class MiniNtp(DatagramProtocol):
    def datagramReceived(self, data, host_and_port):
        for encoding in ['utf8', 'latin1']:
            try:
                d = data.decode(encoding)
                break
            except UnicodeDecodeError:
                print('Failed decoding: {}'.format(encoding))
                pass

        if d and (len(data) < 4 or d[3]!= u'*'): #monlist command
            #bogus packet, discard
            return
        logdata={'NTP CMD': 'monlist'}
        self.transport.getPeer = lambda: IPv4Address('UDP', host_and_port[0], host_and_port[1])
        self.factory.log(logdata=logdata, transport=self.transport)

class CanaryNtp(CanaryService):
    NAME = 'ntp'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('ntp.port', default=123))
        self.logtype = logger.LOG_NTP_MONLIST
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        f = MiniNtp()
        f.factory = self
        return internet.UDPServer(self.port, f, interface=self.listen_addr)
